@@ -22,6 +22,7 @@ __author__ = "Petter Reinholdtsen <pere@hungry.com>"
 import sys
 import apt
 import gc
+import gi
 import fnmatch
 import os
 import urllib
@@ -29,13 +30,14 @@ import errno
 
 pkgdir = "/usr/share/isenkram"
 
+
 def my_modaliases():
     modaliases = set()
     for dirpath, _, filenames in os.walk("/sys/devices"):
         if "modalias" in filenames:
             with open(dirpath + "/modalias") as sysfile:
                 modaliases.add(sysfile.read().strip())
-    with open ("/proc/modules") as procfile:
+    with open("/proc/modules") as procfile:
         line = procfile.readline()
         while line:
             modalias = "lkmodule:%s" % (line.split(" "))[0]
@@ -44,6 +46,7 @@ def my_modaliases():
         procfile.close()
     return list(modaliases)
 
+
 def modalias_match(alias, candidates):
 #    print "modalias_match('%s', %s)" % (alias, candidates)
     for part in candidates.split(')'):
@@ -61,6 +64,7 @@ def modalias_match(alias, candidates):
                 return True
     return False
 
+
 def pkgs_handling_apt_modaliases(modaliaslist):
     cache = apt.Cache()
     cache.open(None)
@@ -72,7 +76,7 @@ def pkgs_handling_apt_modaliases(modaliaslist):
         if version is None:
             continue
         record = version.record
-        if not record.has_key('Modaliases'):
+        if not 'Modaliases' in record:
             continue
         modaliases = record['Modaliases']
         for modalias in modaliaslist:
@@ -85,6 +89,37 @@ def pkgs_handling_apt_modaliases(modaliaslist):
 
     return thepkgs.keys()
 
+
+def pkgs_handling_appstream_modaliases(modaliaslist):
+    thepkgs = {}
+    gi.require_version('AppStream', '1.0')
+    from gi.repository import AppStream
+    db = AppStream.Database()
+    db.open()
+
+    cpts = db.get_all_components()
+    ma_cpts = list()
+    for cpt in cpts:
+        provided = cpt.get_provided_for_kind(AppStream.ProvidedKind.MODALIAS)
+        if provided:
+            ma_cpts.append(cpt)
+
+    for cpt in ma_cpts:
+        prov = cpt.get_provided_for_kind(AppStream.ProvidedKind.MODALIAS)
+        packages = cpt.get_pkgnames()
+        for modalias in modaliaslist:
+            for item in prov.get_items():
+                pkgmodalias = str(item)
+                # Transport modalias to module(alias) format to be
+                # able to use modalias_match().  Perhaps better to
+                # create a new modalias_match() like function without
+                # that formatting requirement.
+                if modalias_match(modalias, 'dummy(' + pkgmodalias + ')'):
+                    for pkg in packages:
+                        thepkgs[pkg] = True
+    return thepkgs.keys()
+
+
 def check_packages_file(pkghash, fileref, modaliaslist):
     line = fileref.readline()
     while line:
@@ -98,6 +133,7 @@ def check_packages_file(pkghash, fileref, modaliaslist):
         line = fileref.readline()
     fileref.close()
 
+
 def pkgs_handling_extra_modaliases(modaliaslist):
     """
 Look up package-hardware mappings from svn and local file
@@ -108,7 +144,7 @@ Look up package-hardware mappings from svn and local file
         f = urllib.urlopen(url)
         check_packages_file(thepkgs, f, modaliaslist)
     except IOError, e:
-        if errno.ENOENT == e.errno: # Most likely lack network connection
+        if errno.ENOENT == e.errno:  # Most likely lack network connection
             pass
     try:
         try:
@@ -124,24 +160,29 @@ Look up package-hardware mappings from svn and local file
                 with open(dirpath + "/modalias") as aliasfile:
                     check_packages_file(thepkgs, aliasfile, modaliaslist)
 
-    except: # Ignore errors
+    except:  # Ignore errors
         pass
 #    print thepkgs
     return thepkgs.keys()
 
+
 def main():
     if 1 < len(sys.argv):
         hwaliases = sys.argv[1:]
     else:
         hwaliases = my_modaliases()
 
+    print "Locating packages supporting this hardware (AppStream):"
+    for pkg in sorted(pkgs_handling_appstream_modaliases(hwaliases)):
+        print "  %s" % pkg
+
     print "Locating packages supporting this hardware (extra, svn/local):"
     for pkg in sorted(pkgs_handling_extra_modaliases(hwaliases)):
-        print "  %s" %pkg
+        print "  %s" % pkg
 
-    print "Locating packages supporting this hardware (APT):"
+    print "Locating packages supporting this hardware (APT control field):"
     for pkg in sorted(pkgs_handling_apt_modaliases(hwaliases)):
-        print "  %s" %pkg
+        print "  %s" % pkg
 
 if __name__ == '__main__':
     main()
